% HuSeiler_14RESS_Matlab.m
%
% This file generates the figures shown in the following submission:
%
% B. Hu and P. Seiler, "Pivotal Decomposition in Reliability
% Analysis of Fault Tolerant Control Systems", submitted to IEEE 
% transactions on Reliability, 2014.
%

%% Initialize Parameters

% Window Sizes
frequency=50;           % Samle frequency, Hz
dt = 1/frequency;       % Sample time, sec
N = 3600/dt;            % Number of samples per hour
N0 = 10;                % Maximum Detection Delay
n=6;                    % component number
% Per-frame actuator failure rate (geometric failure distribution)
MTBF = 3.6e6/2.5;           % Mean-time between failure, sec
q = 1-exp(-dt/MTBF);    % Geometric single-step failure rate, unitless
qhat = 1-(1-q)^N;       % Geometric N-step failure rate, unitless
C=[5 6];                % Single component failure covered by passive FTC
D=[1 2 3 4];            % Single component failure covered by active FTC




%% System unreliability P[T<=N] vs. normalized fault level and threshold, all faults are changing together
fdata = [1 10];       % Normalized fault levels f/sigma, unitless
Hdata = 0:0.1:20;      % Normalized threshold H/sigma, unitless

fdata = 1:0.1:10;

Nf = numel(fdata);
NH = numel(Hdata);
PSN = zeros(Nf,NH);
PFdata = zeros(Nf,NH);
PDdata = zeros(Nf,NH);
for i1=1:Nf
    for i2=1:NH
        % Set normalized fault level and threshold.
        f = fdata(i1);
        H = Hdata(i2);
        
        % Compute per-frame false alarm and detection probability
        rt2 = sqrt(2);
        PF = 1 - erf(H/rt2)+1e-16;
        PD = 1 -( erf((H+f)/rt2) - erf((-H+f)/rt2) )/2;
        
        % Store per-frame false alarm and detection probability
        PFdata(i1,i2) = PF;
        PDdata(i1,i2) = PD;
        
        % Compute probabilities of basic events
        PCat1=1-(1-qhat)^n-n*(1-qhat)^(n-1)*qhat;                      %Compute the mode M_j not in C and D
        PCat2=numel(C)*(1-qhat)^(n-1)*qhat*(1-(1-PF)^(4*N));           %Compute the mode M_j in C
        
        %Compute the mode M_j in D
        PCat3a=numel(D)*(1-qhat)^(n-1)*q*(1-PD)^(N0)*(1-(1-PF)^N*(1-q)^N) / (1-(1-PF)*(1-q));
        PCat3b1=numel(D)*(1-qhat)^(n-1)*qhat;
        PCat3b2=-numel(D)*(1-qhat)^(n-1)*q*(1-(1-PF)^N*(1-q)^N) / (1-(1-PF)*(1-q));
        PCat3b3=-numel(D)*(1-qhat)^(n-1)*PF*qhat/((1-(1-PF)^4));
        PCat3b4=numel(D)*(1-qhat)^(n-1)*PF*q*(1-(1-PF)^(4*N)*(1-q)^(N))/((1-(1-PF)^4)*(1-(1-PF)^4*(1-q)));
        PCat3b5=numel(D)*(1-qhat)^(n-1)*q*(1-(1-PD)^(N0))*(1-(1-PF)^N*(1-q)^N) / (1-(1-PF)*(1-q));
        PCat3b6=-numel(D)*(1-qhat)^(n-1)*PD*q*((1-(1-PD)^N0*(1-PF)^(3*N0))*(1-(1-PF)^(4*N)*(1-q)^(N)))/((1-(1-PF)^4*(1-q))*(1-(1-PD)*(1-PF)^3));
        
        
        % Compute total system failure rate
        %PSN(i1,i2) = PCat1+PCat2+PCat3a+PCat3b1+PCat3b2+PCat3b3+PCat3b4;
        PSN(i1,i2) = PCat1+PCat2+PCat3a+PCat3b1+PCat3b2+PCat3b3+PCat3b4+PCat3b5+PCat3b6;
        PCat(1,i2)=PCat1;
        PCat(2,i2)=PCat2;
        PCat(3,i2)=PCat3a;
        PCat(4,i2)=PCat3b1+PCat3b2+PCat3b3+PCat3b4+PCat3b5+PCat3b6;
    end
end

% Compute multiple (N-step) FDI performance probabilities
PF = PFdata;
PD = PDdata;
PFhat = 1-(1-PFdata).^N;
PDhat = 1-(1-PDdata).^N0;

%Monte Carlo result generated from another code
PSNMC=[0.0120 0.0013 9e-5 0.001 0.0102 0.0105];
HMC=[2.7 5.2 8 10.8 13.2 18];

% Plot PSN vs threshold for different fault sizes
figure(1);
semilogy(Hdata,PSN(1,:),'b',Hdata,PSN(end,:),'r--',HMC,PSNMC,'k*');



xlabel('$H/\sigma$','Interpreter','latex');
 ylabel('$P[T\le N]$','Interpreter','latex');
axis([Hdata(1) Hdata(end) 7e-5 1e-1]);
grid on;

h = legend('$f/\sigma=1$, Eq. (10)','$f/\sigma=10$, Eq. (10)','$f/\sigma=10$, MC');
  set(h,'Interpreter','latex');
set(gca,'Xtick',0:4:20,'XMinorGrid','off');
set(gca,'Ytick',10.^(-4:-2),'YMinorGrid','off');
if exist('garyfyFigure','file')>0
    % Function to clean up plot for paper submissions
    garyfyFigure;
end




%% System unreliability P[T<=N] vs. normalized fault level and threshold, one fault is extremely small
fdata = 1:0.1:10;       % Normalized fault levels f/sigma, unitless
Hdata = 0:0.1:20;      % Normalized threshold H/sigma, unitless
fixedF=10;
fixedH=8.4;
%fdata = 1:0.1:10;

Nf = numel(fdata);
NH = numel(Hdata);
PSN = zeros(Nf,NH);
PFdata = zeros(Nf,NH);
PDdata = zeros(Nf,NH);
for i1=1:Nf
    for i2=1:NH
        % Set normalized fault level and threshold.
        f = fdata(i1);
        H = Hdata(i2);
        
        % Compute per-frame false alarm and detection probability
        rt2 = sqrt(2);
        PF1 = 1 - erf(H/rt2)+1e-16;
        PD1= 1 -( erf((H+f)/rt2) - erf((-H+f)/rt2) )/2;
        PF2 = 1 - erf(fixedH/rt2)+1e-16;
        PD2= 1 -( erf((fixedH+fixedF)/rt2) - erf((-fixedH+fixedF)/rt2) )/2;
        % Store per-frame false alarm and detection probability
        PFdata(i1,i2) = PF1;
        PDdata(i1,i2) = PD1;
        
        % Compute probabilities of basic events
        PCat1=1-(1-qhat)^n-n*(1-qhat)^(n-1)*qhat;                      %M_j not in C and D
        PCat2=numel(C)*(1-qhat)^(n-1)*qhat*(1-(1-PF2)^(3*N)*(1-PF1)^N);           %M_j in C
        
        %M_j in D
        PCat3a=3*(1-qhat)^(n-1)*q*(1-PD2)^(N0)*(1-(1-PF2)^N*(1-q)^N) / (1-(1-PF2)*(1-q))+(1-qhat)^(n-1)*q*(1-PD1)^(N0)*(1-(1-PF1)^N*(1-q)^N) / (1-(1-PF1)*(1-q));
        PCat3b1=numel(D)*(1-qhat)^(n-1)*qhat;
        PCat3b2=-3*(1-qhat)^(n-1)*q*(1-(1-PF2)^N*(1-q)^N) / (1-(1-PF2)*(1-q))-(1-qhat)^(n-1)*q*(1-(1-PF1)^N*(1-q)^N) / (1-(1-PF1)*(1-q));
        PCat3b3=-3*(1-qhat)^(n-1)*PF2*qhat/((1-(1-PF2)^3*(1-PF1)))-(1-qhat)^(n-1)*PF1*qhat/((1-(1-PF1)*(1-PF2)^3));
        PCat3b4=3*(1-qhat)^(n-1)*PF2*q*(1-(1-PF2)^(3*N)*(1-PF1)^N*(1-q)^(N))/((1-(1-PF2)^3*(1-PF1))*(1-(1-PF2)^3*(1-PF1)*(1-q)))+(1-qhat)^(n-1)*PF1*q*(1-(1-PF2)^(3*N)*(1-PF1)^N*(1-q)^(N))/((1-(1-PF2)^3*(1-PF1))*(1-(1-PF2)^3*(1-PF1)*(1-q)));
        PCat3b5=(1-qhat)^(n-1)*q*(1-(1-PD1)^(N0))*(1-(1-PF1)^N*(1-q)^N) / (1-(1-PF1)*(1-q))+3*(1-qhat)^(n-1)*q*(1-(1-PD2)^(N0))*(1-(1-PF2)^N*(1-q)^N) / (1-(1-PF2)*(1-q));
        PCat3b6=-3*(1-qhat)^(n-1)*PD2*q*((1-(1-PD2)^N0*(1-PF2)^(2*N0)*(1-PF1)^N0)*(1-(1-PF1)^N*(1-PF2)^(3*N)*(1-q)^(N)))/((1-(1-PF2)^3*(1-PF1)*(1-q))*(1-(1-PD2)*(1-PF2)^2*(1-PF1)));
        PCat3b7=-(1-qhat)^(n-1)*PD1*q*((1-(1-PD1)^N0*(1-PF2)^(3*N0))*(1-(1-PF1)^N*(1-PF2)^(3*N)*(1-q)^(N)))/((1-(1-PF2)^3*(1-PF1)*(1-q))*(1-(1-PD1)*(1-PF2)^3));
       
        
        % Compute total system failure rate
       
        PSN(i1,i2) = PCat1+PCat2+PCat3a+PCat3b1+PCat3b2+PCat3b3+PCat3b4+PCat3b5+PCat3b6+PCat3b7;
        PCat(1,i2)=PCat1;
        PCat(2,i2)=PCat2;
        PCat(3,i2)=PCat3a;
        PCat(4,i2)=PCat3b1+PCat3b2;
       
    end
end

% Compute multiple (N-step) FDI performance probabilities
PF = PFdata;
PD = PDdata;
PFhat = 1-(1-PFdata).^N;
PDhat = 1-(1-PDdata).^N0;

% Plot PSN vs threshold for different fault sizes
figure(2);
semilogy(Hdata,PSN(1,:),'b',Hdata,PSN(end,:),'r--');
%semilogy(Hdata,PSN(end,:),'r--',Hdata,PCat(1,:),Hdata,PCat(2,:),'k',Hdata,PCat(3,:),'y',Hdata,PCat(4,:),'r');
% xlabel('H_4/\sigma');
% ylabel('P[T <= N]')
% grid on;
% legend('f_4/\sigma=1','f_4/\sigma=10')
axis([Hdata(1) Hdata(end) 7e-5 4e-2]);
xlabel('$H_4/\sigma$','Interpreter','latex');
 ylabel('$P[T\le N]$','Interpreter','latex');

grid on;

h = legend('$f_4/\sigma=1$','$f_4/\sigma=10$');
  set(h,'Interpreter','latex');
set(gca,'Xtick',0:4:20,'XMinorGrid','off');
set(gca,'Ytick',10.^(-4:-2),'YMinorGrid','off');
if exist('garyfyFigure','file')>0
    % Function to clean up plot for paper submissions
    garyfyFigure;
end










%% PSN vs. normalized fault level for optimal threshold for fixed H

% Determine optimal threshold and performance
[PSNOpt,HOptidx] = min(PSN,[],2);
HOpt = Hdata(HOptidx);

for i1=1:Nf
    
        % Set normalized fault level and threshold.
        f = fdata(i1);
        H = HOpt(i1);
        
        % Compute per-frame false alarm and detection probability
        rt2 = sqrt(2);
        PF1 = 1 - erf(H/rt2)+1e-16;
        PD1= 1 -( erf((H+f)/rt2) - erf((-H+f)/rt2) )/2;
        PF2 = 1 - erf(fixedH/rt2)+1e-16;
        PD2= 1 -( erf((fixedH+fixedF)/rt2) - erf((-fixedH+fixedF)/rt2) )/2;
        % Store  false alarm, missed detection and missed isolation probability
        
        PFhat = 1-(1-PF1)^N*(1-PF2)^(3*N);
PM1hat = (1-PD1).^N0;
      PM2hat=0;
for j=1:N0-1
        PM2hat=PM2hat+(1-PF2)^(3*j-3)*PF2^3*(1-PD1)^j*(1-(1-PD1)^(N0-j));
        end
        Phat(i1) = PFhat+PM1hat+PM2hat;       
 end



% Plot optimal PSN vs normalized fault size
figure(3);
subplot(311)
semilogy(fdata,PSNOpt);
xlabel('$f_4/\sigma$','Interpreter','latex');
 ylabel('$P^*[T\le N]$','Interpreter','latex');
 axis([fdata(1) fdata(end) 7e-5 4e-3 ]);
grid on;

% if exist('garyfyFigure','file')>0
%     % Function to clean up plot for paper submissions
%     garyfyFigure;
% end
set(gca,'XMinorGrid','off');
set(gca,'Ytick',10.^(-4:-2),'YMinorGrid','off');

% Plot optimal threshold vs normalized fault size
subplot(312)
plot(fdata,HOpt);
axis([fdata(1) fdata(end) 5 9]);
xlabel('$f_4/\sigma$','Interpreter','latex');
 ylabel('$H_4^*/\sigma$','Interpreter','latex');

grid on;




subplot(313)
semilogy(fdata,Phat);
xlabel('$f_4/\sigma$','Interpreter','latex');
 ylabel('$\hat{P}_{B,4}^*$','Interpreter','latex');
 axis([fdata(1) fdata(end) 1e-10 1]);
grid on;
hold on;
idx = 51;
plot(fdata(idx),Phat(idx),'bs','MarkerSize',10)
hold off
% set(gca,'XMinorGrid','off');
% set(gca,'Ytick',10.^(1:-5),'YMinorGrid','off');
%axis([fdata(1) fdata(end) 5 9]);
%grid on
if exist('garyfyFigure','file')>0
    % Function to clean up plot for paper submissions
    garyfyFigure;
end














